# 
# Coloring count cones of planar graphs
#     Zdenek Dvorak and Bernard Lidicky
#
# The following program is helping with the proof of Lemma 2.3
# It gives all the rays. It does not give the entire proof but does the 
# difficult calculation of cones in a polyhedra.
#
# The following is an expression of colorings as the Kempe configurations
#  AAABC = x_1 + x_2
#  AAABC = x_3 + x_4
#  .....
abc=[]
abc.append(['AAABC',1,2])
abc.append(['AAABC',3,4])
abc.append(['AABAC',1,5])
abc.append(['AABAC',6,7])
abc.append(['AABCA',3,8])
abc.append(['AABCA',6,9])
abc.append(['ABAAC',10,5])
abc.append(['ABAAC',11,12])
abc.append(['ABACA',13,8])
abc.append(['ABACA',11,14])
abc.append(['ABBBC',10,2])
abc.append(['ABBBC',15,16])
abc.append(['ABBCB',13,4])
abc.append(['ABBCB',15,17])
abc.append(['ABCAA',18,9])
abc.append(['ABCAA',19,14])
abc.append(['ABCBB',18,7])
abc.append(['ABCBB',20,17])
abc.append(['ABCCC',19,12])
abc.append(['ABCCC',20,16])


# Next we caluclate xvariables as the count AAABC,...
# and yvariables, which are the kempe variables

ABCtoINT = dict()
yvariables = 0
for a in abc:
    if a[0] not in ABCtoINT:
        nextID = len(ABCtoINT)
        ABCtoINT[a[0]] = nextID
    for j in [1,2]:
        if yvariables < a[j]:
            yvariables = a[j]
#print ABCtoINT, yvariables 

xvariables=len(ABCtoINT)

variables= xvariables+yvariables


# Next we write the constraints of the type  AAABC = x_1 + x_2
# as long vectors
constraints=[]
for a in abc:
    coefficients = [0]*variables
    coefficients[ABCtoINT[a[0]]] = 1
    coefficients[a[1]+xvariables-1] = -1
    coefficients[a[2]+xvariables-1] = -1
    constraints.append([0, coefficients])

# If we want, we could cut the cone and the result will be vertices rather than rays
'''
    coefficients = [1]*xvariables + [0]*yvariables
    constraints.append([100, coefficients])  
    '''

# Variables that are greater or equal zero. We just force the kempe variables to be >= 0
gezero= [0]*xvariables + [1]*yvariables



eqs = []
ieqs = []
for con in constraints:
    x = [-con[0]] + con[1]
    ieqs.append(x)
    ieqs.append([ -z for z in x])

variables= xvariables+ yvariables    
for gez in range(len(gezero)):
    if gezero[gez] == 1:
        v = [0]*(variables+1)
        v[gez+1] = 1
        ieqs.append(v)


P = Polyhedron(ieqs = ieqs)


# For debug 
# print  Pie.Vrepresentation()



# We think this should be all rays!
# TODO: Maybe we should write to each ray which graph it corresponds to? :-(
all_rays=[[1, 0, 0, 0, 0, 1, 1, 0, 0, 0],
          [0, 0, 0, 0, 1, 0, 1, 1, 1, 0],
          [0, 0, 1, 0, 1, 0, 0, 1, 0, 0],
          [0, 0, 0, 1, 1, 1, 1, 0, 0, 0],
          [0, 1, 1, 1, 1, 0, 0, 0, 0, 0],
          [1, 1, 1, 0, 0, 0, 0, 0, 0, 0],
          [0, 1, 0, 1, 0, 0, 0, 0, 1, 1],
          [1, 1, 0, 0, 0, 0, 1, 0, 1, 0],
          [0, 1, 0, 1, 1, 0, 1, 0, 1, 0],
          [0, 0, 0, 0, 0, 0, 0, 1, 1, 1],
          [0, 0, 0, 1, 0, 1, 0, 0, 0, 1],
          [1, 0, 1, 0, 0, 1, 0, 1, 0, 1]]


c = Cone(all_rays)

# Go through all rays of P and test if they contain any extra ray
p_rays=[]
for z in P.ray_generator():
    r = z[0:xvariables]
    p_rays.append(r)
    if not c.contains(r):
        print "Ray",r,"not found!!!"

# Also test if all_rays are indeed in P
Pc = Cone(p_rays)
for r in all_rays:
    if not Pc.contains(r):
        print "Ray",r,"not found!!!"

print "If the above has no error message, all rays are"
for r in all_rays:
    print r
